#include <os.h>

#define INTRS 64

// comon entry point for all interrupt service routines

static void intr_dispatch(int intrno);

static void __declspec(naked) isr(void)
{
  __asm
  {
    push ebx
    push ecx
    push edx
    push esi
    push edi
    push ebp

    push eax
    call intr_dispatch
    add esp, 4

    pop ebp
    pop edi
    pop esi
    pop edx
    pop ecx
    pop ebx
    pop eax

    iretd
  }
}

// generate interrupt service routines

#define ISR(n)                                \
static void __declspec(naked) isr##n(void)    \
{                                             \
  __asm { push eax }                          \
  __asm { mov eax, n }                        \
  __asm { jmp isr }                           \
}

ISR(0)  ISR(1)  ISR(2)  ISR(3)  ISR(4)  ISR(5)  ISR(6)  ISR(7)
ISR(8)  ISR(9)  ISR(10) ISR(11) ISR(12) ISR(13) ISR(14) ISR(15)
ISR(16) ISR(17) ISR(18) ISR(19) ISR(20) ISR(21) ISR(22) ISR(23)
ISR(24) ISR(25) ISR(26) ISR(27) ISR(28) ISR(29) ISR(30) ISR(31)
ISR(32) ISR(33) ISR(34) ISR(35) ISR(36) ISR(37) ISR(38) ISR(39)
ISR(40) ISR(41) ISR(42) ISR(43) ISR(44) ISR(45) ISR(46) ISR(47)
ISR(48) ISR(49) ISR(50) ISR(51) ISR(52) ISR(53) ISR(54) ISR(55)
ISR(56) ISR(57) ISR(58) ISR(59) ISR(60) ISR(61) ISR(62) ISR(63)

// interrupt descriptor table

//struct gate idt[INTRS];
//struct selector idtsel;
struct syspage *syspage = (struct syspage *) SYSPAGE_ADDRESS;


// interrupt dispatch table

intrfunc intrhndlr[INTRS];

// initialize one idt entry

static void init_idt_gate(int intrno, void *handler)
{
  syspage->idt[intrno].offset_low = (unsigned short) (((unsigned int) handler) & 0xFFFF);
  syspage->idt[intrno].selector = SEL_KTEXT;
  syspage->idt[intrno].access = D_PRESENT | D_INT | D_DPL0;
  syspage->idt[intrno].offset_high = (unsigned short) (((unsigned int) handler) >> 16);
}

// default interrupt handler

void default_interrupt_handler(int intrno)
{
  kprintf("Panic: unexpected interrupt %d\n", intrno);
  __asm { hlt };
}

void *virt2phys(void *addr)
{
  pte *pdir = (pte *) PAGEDIR_ADDRESS;
  pte *pt = (pte *) (pdir[L1IDX(addr)] & PT_PFN);
  return (void *) ((pt[L2IDX(addr)] & PT_PFN) + PGOFF(addr));
}

// initialize interrupts

void init_intr()
{
  int i;

  // initialize interrupt dispatch table
  for (i = 0; i < INTRS; i++) intrhndlr[i] = default_interrupt_handler;

  // setup idt
  init_idt_gate(0, isr0);
  init_idt_gate(1, isr1);
  init_idt_gate(2, isr2);
  init_idt_gate(3, isr3);
  init_idt_gate(4, isr4);
  init_idt_gate(5, isr5);
  init_idt_gate(6, isr6);
  init_idt_gate(7, isr7);
  init_idt_gate(8, isr8);
  init_idt_gate(9, isr9);
  init_idt_gate(10, isr10);
  init_idt_gate(11, isr11);
  init_idt_gate(12, isr12);
  init_idt_gate(13, isr13);
  init_idt_gate(14, isr14);
  init_idt_gate(15, isr15);
  init_idt_gate(16, isr16);
  init_idt_gate(17, isr17);
  init_idt_gate(18, isr18);
  init_idt_gate(19, isr19);
  init_idt_gate(20, isr20);
  init_idt_gate(21, isr21);
  init_idt_gate(22, isr22);
  init_idt_gate(23, isr23);
  init_idt_gate(24, isr24);
  init_idt_gate(25, isr25);
  init_idt_gate(26, isr26);
  init_idt_gate(27, isr27);
  init_idt_gate(28, isr28);
  init_idt_gate(29, isr29);
  init_idt_gate(30, isr30);
  init_idt_gate(31, isr31);
  init_idt_gate(32, isr32);
  init_idt_gate(33, isr33);
  init_idt_gate(34, isr34);
  init_idt_gate(35, isr35);
  init_idt_gate(36, isr36);
  init_idt_gate(37, isr37);
  init_idt_gate(38, isr38);
  init_idt_gate(39, isr39);
  init_idt_gate(40, isr40);
  init_idt_gate(41, isr41);
  init_idt_gate(42, isr42);
  init_idt_gate(43, isr43);
  init_idt_gate(44, isr44);
  init_idt_gate(45, isr45);
  init_idt_gate(46, isr46);
  init_idt_gate(47, isr47);
  init_idt_gate(48, isr48);
  init_idt_gate(49, isr49);
  init_idt_gate(50, isr50);
  init_idt_gate(51, isr51);
  init_idt_gate(52, isr52);
  init_idt_gate(53, isr53);
  init_idt_gate(54, isr54);
  init_idt_gate(55, isr55);
  init_idt_gate(56, isr56);
  init_idt_gate(57, isr57);
  init_idt_gate(58, isr58);
  init_idt_gate(59, isr59);
  init_idt_gate(60, isr60);
  init_idt_gate(61, isr61);
  init_idt_gate(62, isr62);
  init_idt_gate(63, isr63);

  // setup idt selector
  //idtsel.limit = INTRS * sizeof(struct segment) - 1;
  //idtsel.dt = idt;

  // load idt 
  //__asm { lidt [idtsel] };
  
  // clear NT (nested task) flag
  //__asm { pushf };
  //__asm { and [esp], 0xFFFFBFFF };
  //__asm { popf };

  // enable interrupts
  __asm { sti };
}

static void intr_dispatch(int intrno)
{
  //kprintf("interrupt %d\n", intrno);

  // call interrupt handler
  intrhndlr[intrno](intrno);
}

void set_interrupt_handler(int intrno, intrfunc f)
{
  intrhndlr[intrno] = f;
}
