#include <os.h>

unsigned int irq_mask = 0xFFFB; // all IRQs disabled initially except cascade

// initialize the 8259 Programmable Interrupt Controller

void init_pic()
{
  _outp(I8259_MSTR_CTRL, I8259_MSTR_ICW1);
  _outp(I8259_SLV_CTRL, I8259_SLV_ICW1);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW2);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW2);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW3);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW3);
  _outp(I8259_MSTR_MASK, I8259_MSTR_ICW4);
  _outp(I8259_SLV_MASK, I8259_SLV_ICW4);
  _outp(I8259_MSTR_MASK, I8259_MSTR_DISABLE);
  _outp(I8259_SLV_MASK, I8259_SLV_DISABLE);
}

// enables irq

void enable_irq(unsigned int irq)
{
  irq_mask &= ~(1 << irq);
  if (irq >= 8) irq_mask &= ~(1 << 2);
	
  _outp(I8259_MSTR_MASK, irq_mask & 0xFF);
  _outp(I8259_SLV_MASK, (irq_mask >> 8) & 0xFF);
}

// disables irq

void disable_irq(unsigned int irq)
{
  irq_mask |= (1 << irq);
  if ((irq_mask & 0xFF00) == 0xFF00) irq_mask |= (1 << 2);
	
  _outp(I8259_MSTR_MASK, irq_mask & 0xFF);
  _outp(I8259_SLV_MASK, (irq_mask >> 8) & 0xFF);
}

// signal end of interrupt to pic

void eoi(unsigned int irq)
{
  if (irq < 8)
    _outp(I8259_MSTR_CTRL, irq + I8259_EOI_BASE);
  else
  {
    _outp(I8259_SLV_CTRL, (irq - 8) + I8259_EOI_BASE);
    _outp(I8259_MSTR_CTRL, I8259_EOI_CAS);
  }
}
