#include <os.h>

#define VERSION "0.0.1"

struct time
{
  int year;
  int month;
  int day;

  int hour;
  int min;
  int sec;
};

void delay()
{
  __asm { nop };
}

int read_cmos_reg(int reg)
{
  unsigned char val;

  _outp(0x70, reg);
  //delay();
  val = _inp(0x71) & 0xFF;

  return val;
}

int read_bcd_cmos_reg(int reg)
{
  int val;

  val = read_cmos_reg(reg);
  return (val >> 4) * 10 + (val & 0x0F);
}

void get_time(struct time *t)
{
  t->year = 2000 + read_bcd_cmos_reg(9);
  t->month = read_bcd_cmos_reg(8);
  t->day = read_bcd_cmos_reg(7);
  t->hour = read_bcd_cmos_reg(4);
  t->min = read_bcd_cmos_reg(2);
  t->sec = read_bcd_cmos_reg(0);
}

void pause()
{
  while (1);
}

void dummy_handler(int intrno)
{
  kprintf("dummy interrupt\n");
}

char *gets(char *buf)
{
  char *p = buf;
  char ch;

  while (1)
  {
    ch = (char) getch();
    if (ch == 8)
    {
      if (p > buf)
      {
        print_char(8);
	p--;
      }

    }
    else
    {
      print_char(ch);
      if (ch == 10) break;
      *p++ = ch;
    }
  }

  *p = 0;
  return buf;
}

void __stdcall start(int reserved1, int reserved2, int reserved3)
{
  char buf[128];
  struct time tm;
  unsigned int basemem;
  unsigned int extmem;

  init_video();
  clear_screen();

  print_string("SanOS version " VERSION " (Build " __DATE__ " " __TIME__ ")\n\n");

  init_pic();
  init_intr();
  init_pit();
  init_keyboard();

  get_time(&tm);
  kprintf("Time is %04d/%02d/%02d %02d:%02d:%02d\n", tm.year, tm.month, tm.day, tm.hour, tm.min, tm.sec);

  basemem = read_cmos_reg(0x15) + (read_cmos_reg(0x16) << 8);
  extmem = read_cmos_reg(0x17) + (read_cmos_reg(0x18) << 8);

  kprintf("%dKB base memory, %dKB extended memory (%d MB memory) \n", basemem, extmem, extmem / 1024 + 1);

  kprintf("\nReady\n");

  while (1)
  {
    gets(buf);
    kprintf("%d input '%s'\n", get_tick_count(), buf);
  }

  pause();
}

void dump_memory(unsigned char *p, int len)
{
  int i;
  char *end;

  end = p + len;
  while (p < end)
  {
    kprintf("%08X ", p);
    for (i = 0; i < 16; i++) kprintf("%02X ", p[i]);
    for (i = 0; i < 16; i++) kprintf("%c", p[i] < 0x20 ? '.' : p[i]);
    kprintf("\n");
    p += 16;
  }
}